/* This include file to the POV raytracer defines some basic triangular geometric functions.

   -------------------------------------------------------------------------------------------        

   #macro draw_line(point_1, point_2, line_radius, line_texture)
   
        just draws a cylinder between the given coordinates with a given texture and radius.
        The cylinder is terminated by two spheres to allow for smooth junctions between
        two lines.                                                                         
        

   #macro draw_vector(point_1, point_2, line_radius, line_texture)
   
        draws an oriented vector arrow between the given coordinates with a given texture and radius.
        

   #macro draw_distance(point_1, point_2, line_radius, line_texture)
      
        draws a double-ended arrow between the given coordinates with a given texture and radius.
        

   #macro mm_paper(filter_factor)
   
        returns the texture parameters for a milimeter spaced paper.
        The filter factor determines the transparency of the paper.
        usage:
                 texture { mm_paper(filter_factor) }       
                 

   #macro fill_slash(filter_factor)

        returns the texture parameters for a slashed filling.
        The filter factor determines the transparency of the pattern.
        usage:
                 texture { fill_slash(filter_factor) }       
   

   #macro fill_backslash(filter_factor)

        returns the texture parameters for a backslashed filling.
        The filter factor determines the transparency of the pattern.
        usage:
                 texture { fill_backslash(filter_factor) }       
   

   #macro fill_x(filter_factor)

        returns the texture parameters for a crossed filling.
        The filter factor determines the transparency of the pattern.
        usage:
                 texture { fill_x(filter_factor) }       

   
   #macro draw_origin(view_vector)                                                                            

        marks the origin of a scene. If not translated or rotated, three arrows are
        drawn in the x, y and z direction respectively. The vectors are differently colored
        (x - red, y - green, z - blue) and a letter at the end of each vector marks its
        direction. The letters are oriented toward the direction of the vector 'view_vector'
        if it's given as a non-zero length vector.
        

   #macro triangle_normal(corner_1, corner_2, corner_3)
      
        returns the surface normal of the triangle given by the three corners 'corner_1',
        'corner_2' and 'corner_3'.
        As POVRay defines a left-handed coordinate system, the normal vector points
        to the direction from which the corner points are seen in clockwise direction.
        

   #macro triangle_H (corner_1, corner_2, corner_3)
        
        returns the vector pointing to the center of mass H of the triangle given 
        by the three corners 'corner_1', 'corner_2' and 'corner_3'.

        
        
   #macro triangle_M (corner_1, corner_2, corner_3)

        returns the vector pointing to the center M of the outer circle of the triangle 
        given by the three corners 'corner_1', 'corner_2' and 'corner_3'.
                                                         
                                                         
   #macro triangle_r_outer (corner_1, corner_2, corner_3)
   
        returns the scalar value of the radius r_o of the outer circle of the triangle 
        given by the three corners 'corner_1', 'corner_2' and 'corner_3'.
         
   
   #macro triangle_area (corner_1, corner_2, corner_3)
       
        returns the scalar value of the area A of the triangle given by the three 
        corners 'corner_1', 'corner_2' and 'corner_3'.
         

   #macro triangle_circumference (corner_1, corner_2, corner_3)
   
        returns the scalar value of the circumference (i.e. the sum of the sides) of the 
        triangle given by the three corners 'corner_1', 'corner_2' and 'corner_3'.


   #macro triangle_r_inner (corner_1, corner_2, corner_3)

        returns the scalar value of the radius r_o of the inner circle of the triangle 
        given by the three corners 'corner_1', 'corner_2' and 'corner_3'.
         

   #macro triangle_W (corner_1, corner_2, corner_3)

        returns the vector pointing to the center W of the inner circle of the triangle 
        given by the three corners 'corner_1', 'corner_2' and 'corner_3'.
                                                         

   #macro parallelize(vector_1, vector_2)
      
        returns the rotation directives to rotate a given vector (or object)
        'vector_1' so that it becomes parallel to the given control vector 'vector_2'.
        It is useful to orient objects that are created in a planar orientation (e.g.
        'TEXT' objects or 'TORUS' objects) into a given spacial direction.
        Usage:
                object { "here comes your object definition....."
                         parallelize (vector_1, vector_2)
                       }                                
                       
 
See the files demo_01.pov and demo_02.pov for some examples.
           
Disclaimer:    
    
                  This macro is free for use together with the POVray raytracer.
          It should be distributed with the files geometry.txt and demo_01.pov and demo_02.pov.

                                                     Uwe Zimmermann, Stockholm, Sweden 1999.
                                                                  
   -------------------------------------------------------------------------------------------     */


#macro draw_line(point_1, point_2, line_radius, line_texture)
  union { cylinder { point_1, point_2
                     line_radius
                   } 
          sphere { point_1, line_radius }         
          sphere { point_2, line_radius }         
          texture { line_texture }
        }
#end           
             
#macro draw_vector(point_1, point_2, line_radius, line_texture)
  union { cylinder { point_1, point_2-vnormalize(point_2-point_1)*9.5*line_radius
                     line_radius
                   }
          cone     { point_2-vnormalize(point_2-point_1)*10*line_radius, 3*line_radius
                     point_2, 0
                   }                   
          texture  { line_texture }
        }  
#end           
                          
#macro draw_distance(point_1, point_2, line_radius, line_texture)
  union { cylinder { point_1+vnormalize(point_2-point_1)*9.5*line_radius, 
                     point_2-vnormalize(point_2-point_1)*9.5*line_radius,
                     line_radius
                   }
          cone     { point_1+vnormalize(point_2-point_1)*10*line_radius, 3*line_radius
                     point_1, 0
                   }                   
          cone     { point_2-vnormalize(point_2-point_1)*10*line_radius, 3*line_radius
                     point_2, 0
                   }                   
          texture  { line_texture }
        }  
#end           
       
#macro mm_paper(filter_factor)
  pigment { image_map { png "mm_paper.png" filter all filter_factor} } 
  finish  { phong 0.0
            specular 0.0
            ambient 0.3 
          }
#end
          
#macro fill_slash(filter_factor)
  pigment { image_map { png "fill_slash.png" filter all filter_factor }
            scale <0.2,0.2,1> } 
  finish { phong 0.0
           specular 0.0
           ambient 0.3 
         }
#end
                              
#macro fill_backslash(filter_factor)
  pigment { image_map { png "fill_backslash.png" filter all filter_factor } 
            scale <0.2,0.2,1> } 
  finish { phong 0.0
           specular 0.0
           ambient 0.3 
         }
#end

#macro fill_x(filter_factor) 
  pigment { image_map { png "fill_x.png" filter all filter_factor }  
            scale <0.2,0.2,1> } 
  finish { phong 0.0
           specular 0.0
           ambient 0.3 
         }
#end
                                                                                                              
#macro arrow(V1, V2)
  #local dV=V2-V1;           
  #local l=vlength(dV);
  #local r=0.05*l;
  union { cylinder { V1+0.2*dV, V2-0.3*dV, r }
          cone     { V2-0.3*dV, 2*r, V2, 0   }                        
          cone     { V1, 2*r, V1+0.2*dV, r   }                        
        }                   
  
#end

#macro draw_origin(view_vector)                                                                            
  union { object { arrow(0.1*x, x) pigment { color Red } }
          object { arrow(0.1*y, y) pigment { color Green } }
          object { arrow(0.1*z, z) pigment { color Blue } }
          text { ttf "arial.ttf" "X" 1, 0
                 texture { pigment { color Red } }     
                 translate <-0.35,-0.35,-0.5>     
                 scale <0.3,0.3,0.05>      
                 #if (vlength(view_vector)=0)
                   parallelize(<0,0,-1>, <0,0,1>)
                 #else  
                   parallelize(<0,0,-1>, view_vector)
                 #end
                 translate <1.2,0,0>
               }     
          text { ttf "arial.ttf" "Y"1, 0
                 texture { pigment { color Green } }     
                 translate <-0.35,-0.35,-0.5>
                 scale <0.3,0.3,0.05>
                 #if (vlength(view_vector)=0)
                   parallelize(<0,0,-1>, <1,0,0>)
                 #else  
                   parallelize(<0,0,-1>, view_vector)
                 #end
                 translate <0,1.2,0>
               }     
          text { ttf "arial.ttf" "Z" 1, 0
                 texture { pigment { color Blue } }     
                 translate <-0.35,-0.35,-0.5>
                 scale <0.3,0.3,0.05>
                 #if (vlength(view_vector)=0)
                   parallelize(<0,0,-1>, <0,1,0>)
                 #else  
                   parallelize(<0,0,-1>, view_vector)
                 #end
                 translate <0,0,1.2>
               }     

        }                             
#end                       
                               
                               
// normal vector of the triangle
#macro triangle_normal(corner_1, corner_2, corner_3)
  vnormalize(vcross((corner_2-corner_1),(corner_3-corner_1)))
#end    
         
// center of mass H of the triangle
#macro triangle_H (corner_1, corner_2, corner_3)
  (corner_1+corner_2+corner_3)/3
#end    
         
// center M of the outer circle
#macro triangle_M (corner_1, corner_2, corner_3)
  (corner_1+corner_2)/2+sqrt(triangle_r_outer(corner_1,corner_2,corner_3)*triangle_r_outer(corner_1,corner_2,corner_3) - 0.25*vlength(corner_2-corner_1)*vlength(corner_2-corner_1))*(vcross(triangle_normal(corner_1, corner_2, corner_3),vnormalize(corner_2-corner_1)))
#end    
         
// radius r_outer of the outer circle                               
#macro triangle_r_outer (corner_1, corner_2, corner_3)
  #local v1=vnormalize(corner_2-corner_1);
  #local v2=vnormalize(corner_3-corner_1);
  #local v3=corner_3-corner_2;
  #local sin_a=vlength(vcross(v1, v2));
  vlength(v3)/(2*sin_a)
#end  

// Area A of the triangle
#macro triangle_area (corner_1, corner_2, corner_3)
  #local v1=(corner_2-corner_1);
  #local v2=(corner_3-corner_1);
  vlength(vcross(v1,v2))/2
#end      
                        
// circumference of the triangle                        
#macro triangle_circumference (corner_1, corner_2, corner_3)
  #local v1=(corner_2-corner_1);
  #local v2=(corner_3-corner_1);
  #local v3=(corner_3-corner_2);
  (vlength(v1)+vlength(v2)+vlength(v3))
#end      
                                
// radius r_inner of the inner circle                               
#macro triangle_r_inner (corner_1, corner_2, corner_3)
  2*triangle_area(corner_1, corner_2, corner_3)/triangle_circumference(corner_1, corner_2, corner_3)
#end       
                            
// center W of the inner circle                                                            
#macro triangle_W (corner_1, corner_2, corner_3)
  #local v1=(corner_2-corner_1);
  #local v2=(corner_3-corner_1);
  #local v3=(corner_3-corner_2);
  #local r_i=triangle_r_inner(corner_1, corner_2, corner_3);
  #local bis_a=vnormalize(vnormalize(v1)+vnormalize(v2));
  #local sin_a2=vlength(vcross(bis_a, vnormalize(v1)));
  corner_1+r_i/sin_a2*bis_a
#end      
                 
                 
// parallelizes two vectors by rotation of vector_1
#macro parallelize(vector_1, vector_2)
  #local v1=vnormalize(vector_1);
  #local v2=vnormalize(vector_2);
  #local psi_1=asin(v1.y);
  #local psi_2=asin(v2.y);
  #if ((v1.x=0)&(v1.z=0))
    #local phi_1=0;   
  #else  
    #local phi_1=atan2(v1.z, v1.x);   
  #end
  #if ((v2.x=0)&(v2.z=0))
    #local phi_2=0;
  #else  
    #local phi_2=atan2(v2.z, v2.x);
  #end
  rotate <0,phi_1,0>*180/pi
  rotate <0,0,psi_2-psi_1>*180/pi
  rotate <0,-phi_2,0>*180/pi
#end  

         
